/* eslint-disable global-require */
const stylelint = {
  lintScope: 'src/**/*.{vue,scss,css}',
  configName: '.stylelintrc.js',
  codeLintScript() { return `stylelint "${this.lintScope}" --config ${this.configName} --fix`; },
  template: './template/stylelint',
  pkg: {
    devDependencies: {
      stylelint: '^15.6.0',
      'postcss-html': '^1.5.0',
      'stylelint-config-recommended-vue': '^1.4.0',
      'stylelint-config-recess-order': '^4.0.0',
      'stylelint-config-sass-guidelines': '^10.0.0',
    },
  },
};
const eslint = {
  lintScope: 'src/**/*.{js,jsx,vue,json}',
  configName: '.eslintrc.js',
  codeLintScript() { return `eslint "${this.lintScope}" --config ${this.configName} --fix --max-warnings=0`; },
  template: './template/eslint',
  pkg: {
    devDependencies: {
      eslint: '^8.7.0',
      'eslint-plugin-jsonc': '^2.7.0',
      'eslint-plugin-vue': '^8.4.0',
      'eslint-plugin-vuejs-accessibility': '^1.1.1', // @vue/eslint-config-airbnb peerDependencies
      '@vue/eslint-config-airbnb': '^6.0.0',
    },
  },
  genLint({ hasTs }) {
    if (hasTs) {
      this.template = './template/ts_eslint';
      this.lintScope = 'src/**/*.{js,jsx,ts,tsx,vue,json,jsonc,json5}';
      Object.assign(this.pkg.devDependencies, {
        typescript: '^4.9.5',
        '@typescript-eslint/eslint-plugin': '^5.55.0',
        '@typescript-eslint/parser': '^5.55.0',
        '@vue/cli-plugin-typescript': '~5.0.8',
        '@vue/eslint-config-typescript': '^9.1.0',
      });
    }

    return {
      pkg: this.pkg,
      template: this.template,
      codeLintScript: this.codeLintScript(),
    };
  },
};
const typelint = {
  lintScope: 'src/**/*.{ts,tsx,vue}',
  configName: 'tsconfig.json',
  codeLintScript() { return 'vue-tsc -noEmit'; },
  template: './template/typelint',
  pkg: {
    devDependencies: {
      'vue-tsc': '^1.2.0',
    },
  },
};
const codeLintConfig = {
  stylelint,
  eslint,
  typelint,
};
const commitlint = {
  conventional: {
    template: './template/commitlint/conventional',
    pkg: {
      scripts: {
        commit: 'git-cz',
      },
      config: {
        commitizen: {
          path: '@commitlint/cz-commitlint',
        },
      },
      devDependencies: {
        '@commitlint/cli': '^16.0.2',
        '@commitlint/config-conventional': '^16.0.0',
        '@commitlint/cz-commitlint': '^16.0.0',
        commitizen: '^4.2.4',
      },
      gitHooks: {
        'commit-msg': 'commitlint -E GIT_PARAMS',
      },
    },
  },
  customizable: {
    template: './template/commitlint/customizable',
    pkg: {
      scripts: {
        commit: 'git-cz',
      },
      config: {
        commitizen: {
          path: 'node_modules/cz-customizable',
        },
      },
      devDependencies: {
        commitizen: '^4.2.4',
        'commitlint-config-cz': '^0.13.3',
        'cz-customizable': '^6.3.0',
        '@commitlint/cli': '^16.1.0',
      },
      gitHooks: {
        'commit-msg': 'commitlint -E GIT_PARAMS',
      },
    },
  },
};

function genLint({ cfg }) {
  return {
    pkg: cfg.pkg,
    template: cfg.template,
    codeLintScript: cfg.codeLintScript(),
  };
}

function hasPkg(api, pkgName) {
  const { pkg } = api.generator;

  return Boolean(pkg.devDependencies[pkgName] || pkg.dependencies[pkgName]);
}

module.exports = (api) => {
  // lint生成选项
  const options = {
    codeLint: ['style', 'es'], // lint种类
    commitlint: true,
    commitConfig: 'conventional', // ["conventional","customizable"]
    lintstaged: true,
  };

  const renderList = [];
  const pkgList = [];

  // .设置hooks
  (function setHooks() {
    if (!hasPkg(api, 'yorkie')) {
      pkgList.push(
        {
          devDependencies: {
            yorkie: '^2.0.0',
          },
        },
      );
    }
  }());

  // .添加lint-staged，假若存在lintstagedrc.js会被覆盖。
  // 单独对不同类型验证重写lint语句，后面看能否优化
  (function setLintstaged() {
    const pkg = {
      gitHooks: {
        'pre-commit': 'lint-staged',
      },
    };

    if (!hasPkg(api, 'lint-staged')) {
      pkg.devDependencies = {
        'lint-staged': '^12.3.2',
      };
    }
    pkgList.push(pkg);
    renderList.push('./template/lint_staged');
  }());

  // .添加提交文件lint配置
  (function setCodeLint() {
    // 添加ts类型检查
    const hasTs = api.hasPlugin('typescript');

    if (hasTs) {
      options.codeLint.push('type');
    }

    options.codeLint.forEach((lintKey) => {
      const cfg = codeLintConfig[`${lintKey}lint`];
      const { pkg, codeLintScript, template } = cfg.genLint
        ? cfg.genLint({ api, hasTs })
        : genLint({ cfg });

      // 添加script语句
      if (codeLintScript) {
        pkg.scripts = {
          [`lint:${lintKey}`]: codeLintScript,
        };
      }

      pkgList.push(pkg);
      renderList.push(template);
    });
    // 添加npm run lint
    pkgList.push({
      scripts: {
        lint: options.codeLint.map((type) => `npm run lint:${type}`).join(' && '),
      },
    });

    // .inject vscode setting
    require('./injectVscode')();
  }());

  // .添加commitlint
  (function setCommitlint() {
    if (!options.commitlint) {
      return;
    }
    const cfg = commitlint[options.commitConfig];

    pkgList.push(cfg.pkg);
    renderList.push(cfg.template);
  }());

  // .render
  if (renderList.length) {
    renderList.forEach((file) => {
      api.render(file);
    });
  }
  // extendPackage
  if (pkgList.length) {
    pkgList.forEach((pkg) => {
      api.extendPackage(pkg);
    });
  }

  const { postinstall } = api.generator.pkg.scripts;

  const binName = 'lint-inject-vscode';

  const injectPkg = {
    scripts: {

    },
  };

  if (!postinstall) {
    injectPkg.scripts.postinstall = binName;
  } else if (!postinstall.includes(binName)) {
    injectPkg.scripts.postinstall = `${postinstall} && ${binName}`;
  }

  api.extendPackage(injectPkg);
};
